package org.apidesign.apifest08.test;

import junit.framework.TestCase;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorFactory;
import org.apidesign.apifest08.currency.CurrencyRate;
import org.apidesign.apifest08.currency.CurrencyRateFactory;

/** There are many currencies around the world and many banks manipulate
 * with more than one or two at the same time. As banks are usually the
 * best paying clients, which is true even in case of your Convertor API,
 * it is reasonable to listen to their requests.
 * <p>
 * The quest for today is to enhance your existing convertor API to hold
 * information about many currencies and allow conversions between any of them.
 * Also, as conversion rates for diferent currencies usually arise from various
 * bank departments, there is another important need. There is a need to
 * compose two convertors into one by merging all the information about
 * currencies they know about.
 */
public class Task2Test extends TestCase {
    public Task2Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    // As in Task1Test, keep in mind, that there are three parts
    // of the whole system:
    // 1. there is someone who knows the current exchange rate
    // 2. there is someone who wants to do the conversion
    // 3. there is the API between 1. and 2. which allows them to communicate
    // 
    // Please backward compatibly enhance your existing API to support following
    // usecases:
    //
    
    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 75 CZK. This is method for the group of users that
     *  knows the exchange rate, and needs to use the API to create objects
     *  with the exchange rate. Anyone shall be ready to call this method without
     *  any other method being called previously. The API itself shall know
     *  nothing about any rates, before this method is called.
     */
    public static Convertor createTripleConvertor() {
        // Rates: 1USD = 15CZK
        // Rates: 1USD = 20SKK
        // Rates: 75CZK = 100SKK
        CurrencyRate usdCzk = CurrencyRateFactory.getInstance().createCurrencyRate("USD", "CZK", 1, 15);
        CurrencyRate usdSkk = CurrencyRateFactory.getInstance().createCurrencyRate("USD", "SKK", 1, 20);
        CurrencyRate czkSkk = CurrencyRateFactory.getInstance().createCurrencyRate("CZK", "SKK", 75, 100);
        return ConvertorFactory.newInstance().createConvertor(usdCzk, usdSkk, czkSkk);        
    }

    /** Define convertor that understands three currencies. Use it.
     */
    public void testConvertorForUSDandCZKandSKK() throws Exception {
        Convertor c = createTripleConvertor();

        // convert $5 to CZK using c:
        // assertEquals("Result is 75 CZK");
        assertEquals("Result is 75 CZK", 75.0, c.convert("USD", "CZK", 5));
        
        // convert $5 to SKK using c:
        // assertEquals("Result is 100 SKK");
        assertEquals("Result is 100 SKK", 100.0, c.convert("USD", "SKK", 5));

        // convert 200SKK to CZK using c:
        // assertEquals("Result is 150 CZK");
        assertEquals("Result is 150 CZK", 150.0, c.convert("SKK", "CZK", 200));

        // convert 200SKK to USD using c:
        // assertEquals("Result is 10 USD");
        assertEquals("Result is 10 USD", 10.0, c.convert("SKK", "USD", 200));
    }

    /** Merge all currency rates of convertor 1 with convertor 2.
     * Implement this using your API, preferably this method just delegates
     * into some API method which does the actual work, without requiring
     * API clients to code anything complex.
     */
    public static Convertor merge(Convertor one, Convertor two) {
        return ConvertorFactory.newInstance().mergeConvertors(one,two);
    }

    /** Join the convertors from previous task, Task1Test and show that it
     * can be used to do reasonable conversions.
     */
    public void testConvertorComposition() throws Exception {
        Convertor c = merge(
            Task1Test.createCZKtoUSD(),
            Task1Test.createSKKtoCZK()
        );

        // convert $5 to CZK using c:
        // assertEquals("Result is 85 CZK");
        assertEquals("Result is 85 CZK", 85.0, c.convert("USD", "CZK", 5));

        // convert $8 to CZK using c:
        // assertEquals("Result is 136 CZK");
        assertEquals("Result is 136 CZK", 136.0, c.convert("USD", "CZK", 8));

        // convert 1003CZK to USD using c:
        // assertEquals("Result is 59 USD");
        assertEquals("Result is 59 USD", 59.0, c.convert("CZK", "USD", 1003));

        // convert 16CZK using c:
        // assertEquals("Result is 20 SKK");
        assertEquals("Result is 20 SKK", 20.0, c.convert("CZK", "SKK", 16));

        // convert 500SKK to CZK using c:
        // assertEquals("Result is 400 CZK");
        assertEquals("Result is 400 CZK", 400.0, c.convert("SKK", "CZK", 500));

        //test exceptions
        Convertor one = Task1Test.createCZKtoUSD();
        Convertor two = Task1Test.createSKKtoCZK();
        Convertor three = Task1Test.createSKKtoCZK();
        try {
            ConvertorFactory.newInstance().mergeConvertors(one,two,three);
            fail();
        } catch (IllegalArgumentException e) {
            //ok
        }

        //test exceptions
        try {
            ConvertorFactory.newInstance().mergeConvertors(c, two);
            fail();
        } catch (IllegalArgumentException e) {
            //ok
        }
        
        //try convertors from version 1
        Convertor v1one = ConvertorFactory.newInstance().createConvertor("CZE", "CZE", 1, 2);        
        assertEquals("CZE->CZE 1:2 10 expects 20", 20.0, v1one.convert("CZE", "CZE", 10));
        try {
            ConvertorFactory.newInstance().mergeConvertors(v1one, two);
            fail();
        } catch (IllegalArgumentException e) {
            //ok
        }

        Convertor v1two = ConvertorFactory.newInstance().createConvertor("EUR", "", 1, 2);
        assertEquals("EUR->'' 1:2 10 expects 20", 20.0, v1two.convert("EUR", "", 10));
        try {
            ConvertorFactory.newInstance().mergeConvertors(v1two, two);
            fail();
        } catch (IllegalArgumentException e) {
            //ok
        }

        Convertor v1three = ConvertorFactory.newInstance().createConvertor("EUR", "", 1, 2);
        assertEquals("''->EUR 1:2 10 expects 5", 5.0, v1three.convert("", "EUR", 10));
        try {
            ConvertorFactory.newInstance().mergeConvertors(v1three, two);
            fail();
        } catch (IllegalArgumentException e) {
            //ok
        }
        
    }
}
